@@ -1,6 +1,6 @@
 ;;; tramp-cache.el --- file information caching for Tramp  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2000, 2005-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2005-2021 Free Software Foundation, Inc.
 
 ;; Author: Daniel Pittman <daniel@inanna.danann.net>
 ;;         Michael Albinus <michael.albinus@gmx.de>
@@ -70,7 +70,8 @@
 ;;   process key retrieved by `tramp-get-process' (the main connection
 ;;   process).  Other processes could reuse these properties, avoiding
 ;;   recomputation when a new asynchronous process is created by
-;;   `make-process'.  Examples are "remote-path" or "device" (tramp-adb.el).
+;;   `make-process'.  Examples are "remote-path",
+;;   "unsafe-temporary-file" or "device" (tramp-adb.el).
 
 ;;; Code:
 
@@ -162,16 +163,20 @@ Return DEFAULT if not set."
     (tramp-message
      key 8 "%s %s %s; inhibit: %s; cache used: %s; cached at: %s"
      file property value remote-file-name-inhibit-cache cache-used cached-at)
+    ;; For analysis purposes, count the number of getting this file attribute.
     (when (>= tramp-verbose 10)
       (let* ((var (intern (concat "tramp-cache-get-count-" property)))
-	     (val (or (numberp (bound-and-true-p var))
-		      (progn
-			(add-hook 'tramp-cache-unload-hook
-				  (lambda () (makunbound var)))
-			0))))
+	     (val (or (and (boundp var) (numberp (symbol-value var))
+			   (symbol-value var))
+		      0)))
 	(set var (1+ val))))
     value))
 
+(add-hook 'tramp-cache-unload-hook
+	  (lambda ()
+	    (dolist (var (all-completions "tramp-cache-get-count-" obarray))
+	      (unintern var obarray))))
+
 ;;;###tramp-autoload
 (defun tramp-set-file-property (key file property value)
   "Set the PROPERTY of FILE to VALUE, in the cache context of KEY.
@@ -186,16 +191,20 @@ Return VALUE."
     ;; We put the timestamp there.
     (puthash property (cons (current-time) value) hash)
     (tramp-message key 8 "%s %s %s" file property value)
+    ;; For analysis purposes, count the number of setting this file attribute.
     (when (>= tramp-verbose 10)
       (let* ((var (intern (concat "tramp-cache-set-count-" property)))
-	     (val (or (numberp (bound-and-true-p var))
-		      (progn
-			(add-hook 'tramp-cache-unload-hook
-				  (lambda () (makunbound var)))
-			0))))
+	     (val (or (and (boundp var) (numberp (symbol-value var))
+			   (symbol-value var))
+		      0)))
 	(set var (1+ val))))
     value))
 
+(add-hook 'tramp-cache-unload-hook
+	  (lambda ()
+	    (dolist (var (all-completions "tramp-cache-set-count-" obarray))
+	      (unintern var obarray))))
+
 ;;;###tramp-autoload
 (defun tramp-flush-file-property (key file property)
   "Remove PROPERTY of FILE in the cache context of KEY."
@@ -229,8 +238,7 @@ Return VALUE."
 ;;;###tramp-autoload
 (defun tramp-flush-file-properties (key file)
   "Remove all properties of FILE in the cache context of KEY."
-  (let* ((file (tramp-run-real-handler
-		#'directory-file-name (list file)))
+  (let* ((file (tramp-run-real-handler #'directory-file-name (list file)))
 	 (truename (tramp-get-file-property key file "file-truename" nil)))
     ;; Unify localname.  Remove hop from `tramp-file-name' structure.
     (setq file (tramp-compat-file-name-unquote file)
@@ -465,11 +473,11 @@ used to cache connection properties of the local machine."
 	;; don't save either, because all other properties might
 	;; depend on the login name, and we want to give the
 	;; possibility to use another login name later on.  Key
-	;; "started" exists for the "ftp" method only, which must be
+	;; "started" exists for the "ftp" method only, which must not
 	;; be kept persistent.
 	(maphash
 	 (lambda (key value)
-	   (if (and (tramp-file-name-p key) value
+	   (if (and (tramp-file-name-p key) (hash-table-p value)
 		    (not (string-equal
 			  (tramp-file-name-method key) tramp-archive-method))
 		    (not (tramp-file-name-localname key))
